<?php

/**
 * file modification
 * @author Logan Cai (cailongqun [at] yahoo [dot] com [dot] cn)
 * @link www.phpletter.com
 * @since 22/April/2007
 *
 */

// utf-8-marker: äöüß


class file {
    var $fileInfo = '';
    var $filePath = '';
    var $fileStat = '';
    var $mask = '0775';
    var $debug = FALSE;
    var $errors = array();


    /**
     * constructor
     *
     * @param string $path the path to a file or folder
     */
    function __construct($path = NULL) {
	if (!is_null($path)) {
	    if (file_exists($path)) {
		$this->filePath = $path;
		if (is_file($this->filePath)) {
		    $this->fileStat = stat($path);
		    $this->fileInfo['size'] = $this->fileStat[7];
		    $this->fileInfo['atime'] = $this->fileStat[8];
		    $this->fileInfo['ctime'] = $this->fileStat[10];
		    $this->fileInfo['mtime'] = $this->fileStat[9];
		    $this->fileInfo['path'] = $path;
		    $this->fileInfo['name'] = basename($path);
		    $this->fileInfo['is_writable'] = $this->isWritable();
		    $this->fileInfo['is_readable'] = $this->isReadable();
		    if (is_image_file($path)) {
			$this->fileInfo['is_image'] = 1;
			$imageSize = getimagesize($path);
			$this->fileInfo['x'] = $imageSize[0];
			$this->fileInfo['y'] = $imageSize[1];
		    } else {
			$this->fileInfo['is_image'] = 0;
		    }
		} elseif (is_dir($this->filePath)) {
		    $this->fileStat = stat($path);
		    $this->fileInfo['name'] = basename($path);
		    $this->fileInfo['path'] = $path;
		    $this->fileInfo['atime'] = $this->fileStat[8];
		    $this->fileInfo['ctime'] = $this->fileStat[10];
		    $this->fileInfo['mtime'] = $this->fileStat[9];
		    $this->fileInfo['is_writable'] = $this->isWritable();
		    $this->fileInfo['is_readable'] = $this->isReadable();
		}
	    } else {
		trigger_error('File does not exist ', E_USER_NOTICE);
	    }
	}
    }


    /**
     * contructor
     *
     * @param string $path
     */
    function file($path = NULL) {
	$this->__construct($path);
    }


    /**
     * check if a file or folder writable
     *
     * @param file path $path
     * @return boolean
     */
    function isWritable($path = NULL) {
	// TODO: is special handling for windows really necessary?
	$path = is_null($path) ? $this->filePath : $path;
	if (DIRECTORY_SEPARATOR == '\\') {
	    $path = slashToBackslash($path);
	    if (is_file($path)) {
		$fp = fopen($path,'ab');
		if ($fp) {
		    fclose($fp);
		    return TRUE;
		}
	    } elseif (is_dir($path)) {
		$path = addTrailingSlash($path);
		$tmp = uniqid(time());
		if (touch($path.$tmp))
		{
		    unlink($path.$tmp);
		    return TRUE;
		}
	    }
	    return FALSE;
	} else {
	    return is_writable(slashToBackslash($path));
	}
    }


    /**
     * Returns true if the files is readable.
     *
     * @return boolean true if the files is readable.
     */
    function isReadable($path = NULL) {
	$path = is_null($path) ? $this->filePath : $path;
	return is_readable(slashToBackslash($path));
    }


    /**
     * change the modified time
     *
     * @param string $path
     * @param string $time
     * @return boolean
     */
    function setLastModified($path = NULL, $time) {
	$path = is_null($path) ? $this->filePath : $path;
	$time = is_null($time) ? time() : $time;
	return touch(slashToBackslash($path), $time);
    }


    /**
     * create a new folder
     *
     * @path the path for the new folder
     * @mask
     * @dirOwner
     * @return boolean
     */
    function mkdir($path = NULL, $mask = NULL, $dirOwner='') {
	$path = is_null($path) ? $this->filePath : $path;
	if (!file_exists($path)) {
	    $mask = is_null($mask) ? $this->mask : $mask;
	    $status = mkdir(slashToBackslash($path));
	    if ($mask) {
		chmod(slashToBackslash($path), intval($mask, 8));
	    }
	    if ($dirOwner) {
		$this->chown(slashToBackslash($path), $dirOwner);
	    }
	    return $status;
	}
	return TRUE;
    }


    /**
     * change the owner of a file or folder
     *
     * @param the file path $path
     * @param  $owner
     */
    function chown($path, $owner) {
	if (!empty($owner)) {
	    $owners = explode(':', $owner);
	    if (!empty($owners[0])) {
		chown($path, $owners[0]);
	    }
	    if (!empty($owners[1])) {
		chgrp($path, $owner[1]);
	    }
	}
    }


/**
     * Copy a file, or recursively copy a folder and its contents
     * @author      Aidan Lister <aidan@php.net>
     * @author      Paul Scott
     * @author	Christoph M. Becker
     * @version     1.0.1
     * @param       string   $source    Source path
     * @param       string   $dest      Destination path
     * @return      bool     Returns TRUE on success, FALSE on failure
     */
    function copyTo($source, $dest) {
	$source = removeTrailingSlash(backslashToSlash($source));
	$dest = removeTrailingSlash(backslashToSlash($dest));
	if (!file_exists($dest) || !is_dir($dest)) {
	    if(!$this->mkdir($dest)) {
		$this->_debug('Unable to create folder ('.$dest.')');
		return FALSE;
	    }
	}
	// Simple copy for a file
	if (is_file($source)) {
	    $dest = $this->getNextAvailableFileName($source, $dest);
	    return copy($source, $dest);
	} elseif (is_dir($source)) {
	    // Loop through the folder
	    $dest = $this->getNextAvailableFileName($source, $dest);
		if (!file_exists($dest)
			|| !is_dir($dest)) {
		    if (!$this->mkdir($dest)) {
			$this->_debug('Unable to create folder ('.$dest.')');
			return FALSE;
		    }
		}
		$handle = opendir($source);
		while (($readdir = readdir($handle)) !== FALSE) {
		    if ($readdir != '.' && $readdir != '..') {
			$path = addTrailingSlash($source).$readdir;
			//var_dump($path, $dest);
			$this->copyTo($path, $dest);
		    }
		}
		closedir($handle);
		return TRUE;
	}
	return FALSE;
    }


    /**
     * get next available file name
     *
     * @param string $fileToMove the path of the file will be moved to
     * @param string $destFolder the path of destination folder
     * @return string
     */
    function getNextAvailableFileName($fileToMove, $destFolder) {
	//$folderPath = addslashes(backslashToSlash(getParentPath($fileToMove)));
	//$destFolder = addTrailingSlash(addslashes(backslashToSlash(getParentPath($destFolder))));
	$destFolder = addTrailingSlash(addslashes(backslashToSlash(($destFolder))));
	$finalPath = $destFolder.basename($fileToMove);
	//var_dump($finalPath);
	if (file_exists($finalPath)) {
	    if (is_file($fileToMove)) {
		$fileExt = getFileExt($fileToMove);
		$fileBaseName = basename($fileToMove, '.'.$fileExt);
		$count = 1;
		while (file_exists($destFolder.$fileBaseName.'_'.$count.'.'.$fileExt)) {
		    $count++;
		}
		$finalPath = $destFolder.$fileBaseName.'_'.$count.'.'.$fileExt;
	    } elseif (is_dir($fileToMove)) {
		$folderName = basename($fileToMove);
		$count = 1;
		while (file_exists($destFolder.$folderName.'_'.$count)) {
		    $count++;
		}
		$finalPath = addTrailingSlash($destFolder.$folderName.'_'.$count);
	    }
	}
	return $finalPath;
    }


    /**
     * get file information
     *
     * @return array
     */
    function getFileInfo() {
	return $this->fileInfo;
    }


    /**
     * close
     *
     */
    function close() {
	$this->fileInfo = null;
	$this->fileStat = null;
    }


    /**
     * delete a file or a folder and all contents within that folder
     *
     * @param string $path
     * @return boolean
     */
    function delete($path = NULL) {
	$path = is_null($path) ? $this->filePath : $path;
	if (file_exists($path)) {
	    if (is_file($path)) {
		return unlink($path);
	    } elseif (is_dir($path)) {
		return $this->__recursive_remove_directory($path);
	    }
	}
	return FALSE;
    }


    /**
     * empty a folder
     *
     * @param string $path
     * @return boolean
     */
    function emptyFolder($path) {
	$path = is_null($path) ? $this->filePath : '';
	if (file_exists($path) && is_dir($path)) {
	    return $this->__recursive_remove_directory($path, true);
	}
	return FALSE;
    }


    /**
     *
     */
    function _debug($info) {
	if ($this->debug) {
	    echo $info.'<br>'."\n";
	} else {
	    $this->errors[] = $info;
	}
    }


    /**
    * recursive_remove_directory( directory to delete, empty )
    * expects path to directory and optional TRUE / FALSE to empty
    * of course PHP has to have the rights to delete the directory
    * you specify and all files and folders inside the directory
    *
    * to use this function to totally remove a directory, write:
    * recursive_remove_directory('path/to/directory/to/delete');
    * to use this function to empty a directory, write:
    *  recursive_remove_directory('path/to/full_directory',TRUE);
    * @param string $directory
    * @param boolean $empty
    * @return boolean
    */
    function __recursive_remove_directory($directory, $empty = FALSE) {
	// TODO: removeTrailingSlash?
	if(substr($directory, -1) == '/') {
	    $directory = substr($directory, 0, -1);
	}
	if (!file_exists($directory) || !is_dir($directory)) {
	    return FALSE;
	} elseif (!is_readable($directory)) {
	    return FALSE;
	} else {
	    $handle = opendir($directory);
	    while (($item = @readdir($handle)) !== FALSE) {
		if ($item != '.' && $item != '..') {
		    $path = $directory.'/'.$item;
		    if (is_dir($path)) {
			$this->__recursive_remove_directory($path);
		    } else {
			unlink($path);
		    }
		}
	    }
	    closedir($handle);
	    if (!$empty) {
		if (!rmdir($directory)) {
		    return FALSE;
		}
	    }
	    return TRUE;
	}
    }

}

?>
